#!/usr/bin/env python
#GPIO input monitor on the Raspberry Pi

import os, pygame, sys
from pygame.locals import *
import RPi.GPIO as GPIO

pygame.init()
os.environ['SDL_VIDEO_WINDOW_POS'] = 'center'
pygame.display.set_caption("GPIO monitor")
screen = pygame.display.set_mode([400,282],0,32)
pinSurface = pygame.Surface((400,282))
#cope with not having the picture of the pins
picture = True
try:
  pinPicture = pygame.image.load("images/Gpio.jpg").convert_alpha()
  pinSurface.blit(pinPicture,[0,0])
except:
    picture = False
    pinSurface.fill((220,220,220)) # make just a grey area
font = pygame.font.Font(None, 28)
# positions of pins on the picture
pinx = [99, 113, 127, 142, 157, 172, 185, 200, 214, 228, 243, 258, 272]
piny = [92, 77]
 #calculate box positions
boxX = [ (box * 24) - 18 for box in range (1,14)]
boxCol = (160,60,0)

def main():
 boardRevision =  GPIO.RPI_REVISION
 #define the pins to use
 if boardRevision == 1:
     inputs = [0,1,4,17,21,22,10,9,11,14,15,18,23,24,25,8,7]
 if boardRevision == 2:    
     inputs = [2,3,4,17,27,22,10,9,11,14,15,18,23,24,25,8,7]
 print "Display the GPIO input pin states"
 print "Escape to stop"
 # use real GPIO numbering
 GPIO.setmode(GPIO.BCM)
 pinouts = [3,5,7,11,13,15,19,21,23,8,10,12,16,18,22,24,26] # position of pins
 inputState = [ 5 for temp in range (0,18)] # blank array for input levels
 for pin in range(0,17):
     GPIO.setup(inputs[pin],GPIO.IN, pull_up_down=GPIO.PUD_UP)
 # draw the background screen
 screen.blit(pinSurface,[0,0])
 text = font.render("Board Revision " + str(boardRevision), True, (0,0,0), (220,220,220) )
 textRect = text.get_rect()
 textRect.topleft = 14, 200
 screen.blit(text, textRect)
 text = font.render("Reading all input pins ", True, (0,0,0), (220,220,220) )
 textRect = text.get_rect()
 textRect.topleft = 14, 230
 screen.blit(text, textRect)

 for pin in range(1,27):
     if (pin & 1) == 1: # if it is an odd number
        pygame.draw.line(screen,boxCol,(boxX[(pin-1)/2] , 38),(pinx[(pin-1)/2],piny[1]),2)
        pygame.draw.rect(screen,(180,180,180), (boxX[(pin-1)/2], 12, 20,26), 0)
        pygame.draw.rect(screen,boxCol, (boxX[(pin-1)/2], 12, 20,26), 1)
     else:
        pygame.draw.line(screen,boxCol,(boxX[(pin-1)/2] , 170),(pinx[(pin-1)/2],piny[0]),2)
        pygame.draw.rect(screen,(180,180,180), (boxX[(pin-1)/2], 170, 20,26), 0)
        pygame.draw.rect(screen,boxCol, (boxX[(pin-1)/2], 170, 20,26), 1)
 pygame.display.update()

 # Display fixed pins
 showLogicState("3", 1) # logic state, pin number
 showLogicState("3", 17)
 showLogicState("5", 2)
 showLogicState("5", 4)
 showLogicState("G", 6)
 showLogicState("G", 9)
 showLogicState("G", 25)
 showLogicState("G", 14)
 showLogicState("G", 20)
            
 while True: # do forever
   checkForEvent() # see if we have a quit event
   for check in range(0,17):
     if(GPIO.input(inputs[check])):
        latestState = 1
     else:
        latestState = 0
     if(latestState != inputState[check]):
         showLogicState(str(latestState),pinouts[check])
         inputState[check] = latestState
 #end of main loop

# Function definitions
def showLogicState(state, pin):
    textCol = (0,0,0) # default black
    if state == "0":
       textCol = (0,0,230)
    if state == "1":
       textCol = (230,0,0)
    text = font.render(state, True, textCol, (180,180,180) )
    textRect = text.get_rect()
    textRect.topleft = boxX[(pin-1)/2] + 4, 14 + 158 * (pin & 1)
    screen.blit(text, textRect)
    pygame.display.update()

def terminate():
    print "Closing down please wait"
    pygame.quit()
    sys.exit()
    
def checkForEvent():
    #print "checking for quit"
    for event in pygame.event.get(QUIT): # get all the QUIT events
        terminate() # terminate if any QUIT events are present
    for event in pygame.event.get(KEYUP): # get all the KEYUP events
        if event.key == K_ESCAPE:
            terminate() # terminate if the KEYUP event was for the Esc key

if __name__ == '__main__':
    main()
